#include <fcntl.h>
#include <sys/stat.h>
#include <stddef.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
//----------------------------------------------------------------------
static void cat_file_descriptor (int fd);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
    int         i;
    int         fd;
    struct stat file_status;
    //
    // Check if the input comes from standard input.
    //
    if (argc < 2)
      {
        cat_file_descriptor (STDIN_FILENO);
        exit (0);
      }
    //
    // There is at least an argument: scan them.
    //
    for(i = 1; i < argc; i++)
      {
        //
        // Verify if the file exists.
        //
        if (stat(argv[i], &file_status) != 0)
          {
            fprintf (stderr, "File \"%s\" does not exist!\n",
                     argv[i]);
            continue;
          }
        //
        // File exists: check the file type.
        //
        if (S_ISDIR (file_status.st_mode))
          {
            fprintf (stderr, "Cannot \"cat\" "
                             "\"%s\": it is a directory!\n",
                     argv[i]);
            continue;
          }
        //
        // File exists and can be "cat"ed.
        //
        fd = open (argv[i], O_RDONLY);
        if (fd >= 0)
          {
            cat_file_descriptor (fd);
            close (fd);
          }
        else
          {
            perror (NULL);
            exit (1);
          }
      }
    return (0);
}
//----------------------------------------------------------------------
static void
cat_file_descriptor (int fd)
{
    ssize_t count;
    char buffer[BUFSIZ];

    for (;;)
      {
        count = read (fd, buffer, (size_t) BUFSIZ);
        if (count > 0)
          {
            write (STDOUT_FILENO, buffer, (size_t) count);
          }
        else
          {
            break;
          }
      }
}

