#include <sys/os16.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
//----------------------------------------------------------------------
static void usage         (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
    int          signal;
    int          pid;
    int          a;             // Index inside arguments.
    int          option_s = 0;
    int          option_l = 0;
    int          opt;
    extern char *optarg;
    extern int   optopt;
    //
    // There must be at least an option, plus the program name.
    //
    if (argc < 2)
      {
        usage ();
        return (1);
      }
    //
    // Check for options.
    //
    while ((opt = getopt (argc, argv, ":ls:")) != -1)
      {
        switch (opt)
          {
            case 'l':
                option_l = 1;
                break;
            case 's':
                option_s = 1;
                //
                // In that case, there must be at least three arguments:
                // the option, the signal and the process id.
                //
                if (argc < 4)
                  {
                    usage ();
                    return (1);
                  }
                //
                // Argument numbers are ok. Check the signal.
                //
                if (strcmp (optarg, "HUP") == 0)
                  {
                    signal = SIGHUP;
                  }
                else if (strcmp (optarg, "INT") == 0)
                  {
                    signal = SIGINT;
                  }
                else if (strcmp (optarg, "QUIT") == 0)
                  {
                    signal = SIGQUIT;
                  }
                else if (strcmp (optarg, "ILL") == 0)
                  {
                    signal = SIGILL;
                  }
                else if (strcmp (optarg, "ABRT") == 0)
                  {
                    signal = SIGABRT;
                  }
                else if (strcmp (optarg, "FPE") == 0)
                  {
                    signal = SIGFPE;
                  }
                else if (strcmp (optarg, "KILL") == 0)
                  {
                    signal = SIGKILL;
                  }
                else if (strcmp (optarg, "SEGV") == 0)
                  {
                    signal = SIGSEGV;
                  }
                else if (strcmp (optarg, "PIPE") == 0)
                  {
                    signal = SIGPIPE;
                  }
                else if (strcmp (optarg, "ALRM") == 0)
                  {
                    signal = SIGALRM;
                  }
                else if (strcmp (optarg, "TERM") == 0)
                  {
                    signal = SIGTERM;
                  }
                else if (strcmp (optarg, "STOP") == 0)
                  {
                    signal = SIGSTOP;
                  }
                else if (strcmp (optarg, "TSTP") == 0)
                  {
                    signal = SIGTSTP;
                  }
                else if (strcmp (optarg, "CONT") == 0)
                  {
                    signal = SIGCONT;
                  }
                else if (strcmp (optarg, "CHLD") == 0)
                  {
                    signal = SIGCHLD;
                  }
                else if (strcmp (optarg, "TTIN") == 0)
                  {
                    signal = SIGTTIN;
                  }
                else if (strcmp (optarg, "TTOU") == 0)
                  {
                    signal = SIGTTOU;
                  }
                else if (strcmp (optarg, "USR1") == 0)
                  {
                    signal = SIGUSR1;
                  }
                else if (strcmp (optarg, "USR2") == 0)
                  {
                    signal = SIGUSR2;
                  }
                else
                  {
                    fprintf (stderr, "Unknown signal %s.\n", optarg);
                    return (1);
                  }
                break;
            case '?':
                fprintf (stderr, "Unknown option -%c.\n", optopt);
                usage ();
                return (1);
                break;
            case ':':
                fprintf (stderr, "Missing argument for option -%c\n",
                         optopt);
                usage ();
                return (1);
                break;
            default:
                fprintf (stderr, "Getopt problem: unknown option %c\n",
                         opt);
                return (1);
          }
      }
    //
    //
    //
    if (option_l && option_s)
      {
        fprintf (stderr, "Options \"-l\" and \"-s\" together ");
        fprintf (stderr, "are incompatible.\n");
        usage ();
        return (1);
      }
    //
    // Option "-l".
    //
    if (option_l)
      {
        printf ("HUP ");
        printf ("INT ");
        printf ("QUIT ");
        printf ("ILL ");
        printf ("ABRT ");
        printf ("FPE ");
        printf ("KILL ");
        printf ("SEGV ");
        printf ("PIPE ");
        printf ("ALRM ");
        printf ("TERM ");
        printf ("STOP ");
        printf ("TSTP ");
        printf ("CONT ");
        printf ("CHLD ");
        printf ("TTIN ");
        printf ("TTOU ");
        printf ("USR1 ");
        printf ("USR2 ");
        printf ("\n");
      }
    //
    // Option "-s".
    //
    if (option_s)
      {
        //
        // Scan arguments.
        //
        for (a = 3; a < argc; a++)
          {
            //
            // Get PID.
            //
            pid = atoi (argv[a]);
            if (pid > 0)
              {
                //
                // Kill.
                //
                if (kill (pid, signal) < 0)
                  {
                    perror (argv[a]);
                  }
              }
            else
              {
                fprintf (stderr, "Invalid PID %s.", argv[a]);
              }
          }
      }
    //
    // All done.
    //
    return (0);
}
//----------------------------------------------------------------------
static void
usage (void)
{
    fprintf (stderr, "Usage: kill -s SIGNAL_NAME PID...\n");
    fprintf (stderr, "       kill -l\n");
}
