#include <sys/os16.h>
#include <kernel/devices.h>
#include <sys/types.h>
#include <errno.h>
#include <kernel/memory.h>
#include <kernel/ibm_i86.h>
#include <kernel/proc.h>
#include <string.h>
#include <signal.h>
#include <kernel/k_libc.h>
#include <ctype.h>
#include <kernel/tty.h>
//----------------------------------------------------------------------
ssize_t
dev_tty (pid_t pid, dev_t device, int rw, off_t offset, void *buffer,
         size_t size, int *eof)
{
    uint8_t *buffer08 = (uint8_t *) buffer;
    ssize_t  n;
    proc_t  *ps;
    //
    // Get process. Variable `ps' will be `NULL' if the process ID is
    // not valid.
    //
    ps = proc_reference (pid);
    //
    // Convert `DEV_TTY' with the controlling terminal for the process.
    //
    if (device == DEV_TTY)
      {
        device = ps->device_tty;
        //
        // As a last resort, use the generic `DEV_CONSOLE'.
        //
        if (device == 0 || device == DEV_TTY)
          {
            device = DEV_CONSOLE;
          }
      }
    //
    // Convert `DEV_CONSOLE' to the currently active console.
    //
    if (device == DEV_CONSOLE)
      {
        device = tty_console ((dev_t) 0);
        //
        // As a last resort, use the first console: `DEV_CONSOLE0'.
        //
        if (device == 0 || device == DEV_TTY)
          {
            device = DEV_CONSOLE0;
          }
      }
    //
    // Read or write.
    //
    if (rw == DEV_READ)
      {
        for (n = 0; n < size; n++)
          {
            buffer08[n] = tty_read (device);
            if (buffer08[n] == 0)
              {
                //
                // If the pid is not the kernel, should put the process
                // to sleep, waiting for the key.
                //
                if (pid == 0 || ps == NULL)
                  {
                    //
                    // For the kernel there is no sleep and for an
                    // unidentified process, either.
                    //
                    break;
                  }
                //
                // Put the process to sleep.
                //
                ps->status         = PROC_SLEEPING;
                ps->ret            = 0;
                ps->wakeup_events  = WAKEUP_EVENT_TTY;
                ps->wakeup_signal  = 0;
                ps->wakeup_timer   = 0;
                //
                break;
              }
            //
            // Check for control characters.
            //
            if (buffer08[n] == 0x04)    // EOT
              {
                //
                // Return EOF.
                //
                *eof = 1;
                break;
              }
            //
            // At this point, show the character on screen, even if it
            // is not nice. It is necessary to show something, because
            // the tty handling is very poor and the library for line
            // input, calculate cursor position based on the characters
            // received.
            //
            tty_write (device, (int) buffer08[n]);
          }
      }
    else
      {
        for (n = 0; n < size; n++)
          {
            tty_write (device, (int) buffer08[n]);
          }
      }
    return (n);
}
