#include <kernel/proc.h>
#include <kernel/k_libc.h>
#include <errno.h>
//----------------------------------------------------------------------
int
fd_close (pid_t pid, int fdn)
{
    inode_t           *inode;
    file_t            *file;
    fd_t *fd;
    //
    // Get file descriptor.
    //
    fd = fd_reference (pid, &fdn);
    if (fd == NULL                ||
        fd->file == NULL          ||
        fd->file->inode == NULL )
      {
        errset (EBADF);         // Bad file descriptor.
        return (-1);
      }
    //
    // Get file.
    //
    file = fd->file;
    //
    // Get inode.
    //
    inode = file->inode;
    //
    // Reduce references inside the file table item
    // and remove item if it reaches zero.
    //
    file->references--;
    if (file->references == 0)
      {
        file->oflags = 0;
        file->inode  = NULL;
        //
        // Put inode, because there are no more file references.
        //
        inode_put (inode);
      }
    //
    // Remove file descriptor.
    //
    fd->fl_flags = 0;
    fd->fd_flags = 0;
    fd->file  = NULL;
    //
    //
    //
    return (0);
}
