#include <kernel/proc.h>
#include <kernel/k_libc.h>
#include <errno.h>
//----------------------------------------------------------------------
off_t
fd_lseek (pid_t pid, int fdn, off_t offset, int whence)
{
    inode_t           *inode;
    file_t            *file;
    fd_t *fd;
    off_t              test_offset;
    //
    // Get file descriptor.
    //
    fd = fd_reference (pid, &fdn);
    if (fd == NULL                ||
        fd->file == NULL          ||
        fd->file->inode == NULL )
      {
        errset (EBADF);                 // Bad file descriptor.
        return (-1);
      }
    //
    // Get file table item.
    //
    file = fd->file;
    //
    // Get inode.
    //
    inode = file->inode;
    //
    // Change position depending on the `whence' parameter.
    //
    if (whence == SEEK_SET)
      {
        if (offset < 0)
          {
            errset (EINVAL);                    // Invalid argument.
            return ((off_t) -1);
          }
        else
          {
            fd->file->offset = offset;
          }
      }
    else if (whence == SEEK_CUR)
      {
        test_offset  = fd->file->offset;
        test_offset += offset;
        if (test_offset < 0)
          {
            errset (EINVAL);                    // Invalid argument.
            return ((off_t) -1);
          }
        else
          {
            fd->file->offset = test_offset;
          }
      }
    else if (whence == SEEK_END)
      {
        test_offset  = inode->size;
        test_offset += offset;
        if (test_offset < 0)
          {
            errset (EINVAL);                    // Invalid argument.
            return ((off_t) -1);
          }
        else
          {
            fd->file->offset = test_offset;
          }
      }
    else
      {
        errset (EINVAL);                        // Invalid argument.
        return ((off_t) -1);
      }
    //
    // Return the new file position.
    //
    return (fd->file->offset);
}
