#ifndef _KERNEL_MEMORY_H
#define _KERNEL_MEMORY_H       1

#include <stdint.h>
#include <stddef.h>
#include <sys/types.h>
//----------------------------------------------------------------------
#define MEM_BLOCK_SIZE     256  // 0x0100
#define MEM_MAX_BLOCKS    2560  // 655360/256 = 0xA0000/0x0100 = 0x0A00

extern uint16_t mb_table[MEM_MAX_BLOCKS/16]; // Memory blocks map.
//----------------------------------------------------------------------
typedef unsigned long int addr_t;
typedef unsigned int      segment_t;
typedef unsigned int      offset_t;
//----------------------------------------------------------------------
typedef struct {
    addr_t    address;
    segment_t segment;
    size_t    size;
} memory_t;
//----------------------------------------------------------------------
addr_t    address       (segment_t segment, offset_t offset);
//----------------------------------------------------------------------
uint16_t *mb_reference  (void);
ssize_t   mb_alloc      (addr_t address, size_t size);
void      mb_free       (addr_t address, size_t size);
int       mb_alloc_size (size_t size, memory_t *allocated);
//----------------------------------------------------------------------
void      mem_copy      (addr_t orig, addr_t dest, size_t size);
size_t    mem_read      (addr_t start, void *buffer, size_t size);
size_t    mem_write     (addr_t start, void *buffer, size_t size);
//----------------------------------------------------------------------

#endif
