#include <dirent.h>
#include <fcntl.h>
#include <const.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stddef.h>
//----------------------------------------------------------------------
DIR *
opendir (const char *path)
{
    int   fdn;
    int   d;
    DIR  *dp;
    struct stat file_status;
    //
    // Function `opendir()' is used only for reading.
    //
    fdn = open (path, O_RDONLY);
    //
    // Check the file descriptor returned.
    //
    if (fdn < 0)
      {
        //
        // The variable `errno' is already set:
        //   EINVAL
        //   EMFILE
        //   ENFILE
        //
        errset (errno);
        return (NULL);
      }
    //
    // Set the `FD_CLOEXEC' flag for that file descriptor.
    //
    if (fcntl (fdn, F_SETFD, FD_CLOEXEC) != 0)
      {
        //
        // The variable `errno' is already set:
        //   EBADF
        //
        errset (errno);
        close (fdn);
        return (NULL);
      }
    //
    //
    //
    if (fstat (fdn, &file_status) != 0)
      {
        //
        // Error should be already set.
        //
        errset (errno);
        close (fdn);
        return (NULL);
      }
    //
    // Verify it is a directory
    //
    if (!S_ISDIR(file_status.st_mode))
      {
        //
        // It is not a directory!
        //
        close (fdn);
        errset (ENOTDIR);               // Is not a directory.
        return (NULL);
      }
    //
    // A valid file descriptor is available: must find a free
    // `_directory_stream[]' slot.
    //
    for (d = 0; d < DOPEN_MAX; d++)
      {
        if (_directory_stream[d].fdn < 0)
          {
            //
            // Found a free slot: set it up.
            //
            dp = &(_directory_stream[d]);
            dp->fdn   = fdn;
            //
            // Return the directory pointer.
            //
            return (dp);
          }
      }
    //
    // If we are here, there was no free directory slot available.
    //
    close  (fdn);
    errset (EMFILE);                    // Too many file open.
    return (NULL);
}
